// rgtp.js -- implements the client side of RGTP
// $Header: /cvs/archangel/src/archangel/content/rgtp.js,v 1.4 2003/02/02 23:54:51 marnanel Exp $
//
// Copyright (c) 2003 Thomas Thurman
// thomas@thurman.org.uk
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have be able to view the GNU General Public License at 
// http://www.gnu.org/copyleft/gpl.html ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

////////////////////////////////////////////////////////////////

var main_rgtp = 0; // icky, but it'll have to do for now

////////////////////////////////////////////////////////////////

function eighthex(number) {
	var hex =  number.toString(16);
	while (hex.length<8) hex = '0'+hex;
	return hex;
}

////////////////////////////////////////////////////////////////

function trim(str) { return str.replace(/\s*$/,''); }

//////////////////////////////////////////////////////////////////

RGTPConnection.prototype.send =
function rc_send(text) {
	this.server.write(text+'\r\n');
}

//////////////////////////////////////////////////////////////////

RGTPConnection.prototype.handleBlock =
function rc_handleBlock(block) {
	try {
		if (this.partialLines) {
			this.partialLines = this.partialLines + block;
		} else {
			this.partialLines = block;
		}

		n = 0;

		while (n!=-1) {
			n = this.partialLines.indexOf('\r\n');
			if (n!=-1) {
				var current = this.partialLines.substring(0, n);
				dump(current);
				if (this.handlingFile) {
					if (current=='.') {
						this.handlingFile = 0;
						this.handleLine(-1, current);
					} else {
						this.handleLine(0, current);
					}
				} else {
					this.handleLine(
						current.substring(0,3),
						current.substring(4));
	
					if (current.substring(0,3) == '250')
						this.handlingFile = 1;
				}
				this.partialLines = this.partialLines.substring(n+2);
			}
		}
	} catch (e) {
		alert('RGTP internal error: '+e);
		throw e;
	}

}

//////////////////////////////////////////////////////////////////

RGTPConnection.prototype.handleInitialConnection =
function rc_handleInitialConnection(code, text) {
	if (code=='230') {
		this.access = 0;
		this.send('USER guest'); // for now, anyway
	} else if (code=='231') {
		this.access = 1;

		// Okay, so we're in.
		//
		// The next task for us is to get hold of the
		// index. If we've seen some things already,
		// then we'll have stored "maxknown", the highest
		// sequence number we know.

		maxknown = this.node.getAttribute('maxknown');
		if (maxknown) {
			alert('Requesting partial index.');
			// No point warning them about long
			// download times; this should only take a
			// few seconds, if that.

			this.send('INDX #'+maxknown);
		} else {
			alert('Archangel must now download the RGTP index. ' +
				'The download may take a few minutes, since '+
				'the index is typically half a megabyte long.');
			this.send('INDX');
		}

		this.handleLine = this.handleIndex;
	} else
		throw "weird code in lobby";
}

//////////////////////////////////////////////////////////////////

var mostRecentlySeenChild = 0;

RGTPConnection.prototype.handleIndex =
function rc_handleIndex(code, text) {

	if (code==0) {
		var typecode = text.charAt(103);
		var itemid = text.substring(18,26);
		var author = trim(text.substring(27,101));
		var datestamp = eval('0x'+text.substring(9,17));
		var sequence = eval('0x'+text.substring(0,8));

		this.node.setAttribute('maxknown', eighthex(sequence+1));

		if (mostRecentlySeenChild) {

			// Then we've just seen a C, so this is an F.

			if (typecode!='F')
				throw "F must follow C in the index!";

			var parent = this.node.getNode(itemid);
			var child = mostRecentlySeenChild;

			parent.setAttribute('child', child.getSubject());
			child.setAttribute('parent', parent.getSubject());
				
			mostRecentlySeenChild = 0;

		} else if (typecode=='R') {
			var itemnode = this.node.getNode(itemid);

			itemnode.setAttribute('known', eighthex(sequence));
			itemnode.setAttribute('last-by', author);
			itemnode.setAttribute('last-date', datestamp);
			itemnode.setAttribute('replies',
				itemnode.getAttribute('replies')*1+1);
		} else if (typecode=='I' || typecode=='C') {
			var subject = trim(text.substring(105));
			var itemnode = this.node.addNode(itemid);

			itemnode.setAttribute('subject', subject);
			itemnode.setAttribute('first-by', author);
			itemnode.setAttribute('last-by', author);
			itemnode.setAttribute('first-date', datestamp);
			itemnode.setAttribute('last-date', datestamp);
			itemnode.setAttribute('known', eighthex(sequence));
			itemnode.setAttribute('replies', 0);

			// For "C", we also need to store this item node so
			// that we can set the parent-child relationship
			// correctly. The next record must be an "F",
			// representing the parent item of this one.

			if (typecode=='C')
				mostRecentlySeenChild = itemnode;
		}

	} else if (code==-1) {
		alert('Index loaded OK.');
		this.node.flush();
		populateTree();
	} else if (code!=250)
		throw "weird code while reading index";
}

////////////////////////////////////////////////////////////////

RGTPConnection.prototype.open =
function rc_open() {
	this.handleLine = this.handleInitialConnection;
	this.handlingFile = 0;

	this.server = new CConnection(this.host, this.port);
	this.server.onRead = function(b) { main_rgtp.handleBlock(b); }
	this.server.open();
}

////////////////////////////////////////////////////////////////

RGTPConnection.prototype.getItem = function (itemid) {
	this.send('ITEM '+itemid);
	this.handleLine = this.handleItem;
}

////////////////////////////////////////////////////////////////

RGTPConnection.prototype.handleItem = function (code, text) {
	if (code==0) {
		receivedItemLine(text);
	} else if (code==-1) {
		// it's all over
		this.node.flush();
	} else if (code!=250)
		throw "weird code while reading item";
}

////////////////////////////////////////////////////////////////

// Returns a connection to an RGTP server.

function RGTPConnection(rdf_node) {

	this.node = rdf_node;
	this.host = rdf_node.getAttribute('host');
	this.port = rdf_node.getAttribute('port');
	this.partialLines = '';

	main_rgtp = this;

	return this;
}



